package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	/**
	 * ...
	 * @author Time
	 */
	public class Bat extends Entity
	{
		[Embed(source = "data/Bat.png")] private var imgBat:Class;
		private var sprBat:Spritemap = new Spritemap(imgBat, 4, 4);
		
		public static const myScale:Number = Game.scale / 2;
		public static const myLayer:int = 0;
		
		private const hitCounterMax:int = 36;
		private const hitCounterInterval:int = 12;
		private var hitCounter:int = 0;
		private const hitColor:uint = 0x000000;
		private const hitsMax:int = 2;
		private var hits:int = 0;
		
		private const accel:Point = new Point(0.1, 0.2);
		private const friction:Number = 0.025;
		private const maxSpeed:Point = new Point(5, 8);
		private var speed:Number = 0;
		private var angle:Number = 0;
		
		private var direction:int = 1;		
		private var roosted:Boolean = true;
		
		public function Bat(_x:int, _y:int) 
		{
			super(_x, _y, sprBat);
			
			sprBat.centerOO();
			sprBat.centerOrigin();
			sprBat.add("Fly", [0, 1, 2, 3], 18);
			sprBat.add("Sit", [0]);
			sprBat.play("Sit");
			
			sprBat.scaleX = myScale;
			sprBat.scaleY = myScale;
			
			setHitbox(4 * myScale, 3 * myScale, 2 * myScale, myScale);
			type = "Bat";
			
			layer = myLayer;
		}
		
		override public function update():void
		{
			hitStep();
			AI();
			movementValues();
			move();
			spriteValues();
		}
		
		private function hitStep():void
		{
			if (hitCounter > 0)
			{
				if (hitCounter % hitCounterInterval == 0)
				{
					if ((graphic as Image).color != hitColor)
					{
						(graphic as Image).color = hitColor;
					}
					else
					{
						(graphic as Image).color = 0xFFFFFF;
					}
				}
				hitCounter--;
			}
			else
			{
				(graphic as Image).color = 0xFFFFFF;
			}
		}
		
		public function getHurt():void
		{
			if (hitCounter <= 0)
			{
				hits++;
				if (hits >= hitsMax)
				{
					FP.world.remove(this);
				}
				hitCounter = hitCounterMax;
			}
		}
		
		private function AI(range:int = -1):void
		{
			var player:Player = FP.world.nearestToEntity("Player", this) as Player;
			
			if (player)
			{
				const randomMovementAngle:Number = Math.PI / 180 * 150;
				
				var distanceToPlayer:int = FP.distance(x, y, player.x, player.y);
				var seePlayer:Boolean = !FP.world.collideLine("Solid", x, y, player.x, player.y);
				var inRange:Boolean = distanceToPlayer <= range || range == -1;
				var angleToPlayer:Number = -FP.angle(x, y, player.x, player.y) / 180 * Math.PI;
				
				if (seePlayer && inRange)
				{
					if (roosted)
					{
						roosted = false;
						speed = 2;
						angle = Math.PI * 3 / 2;
					}
				}
				if (!roosted)
				{
					angle += FP.angle_difference(angleToPlayer + 2 * randomMovementAngle * Math.random() - randomMovementAngle, angle) / 3;
				}
			}
		}
		
		private function movementValues():void
		{
			if (Math.cos(angle) != 0)
			{
				direction = FP.sign(Math.cos(angle));
			}
		}
		
		private function move():void
		{
			var i:int;
			var sX:Number = speed * Math.cos(angle);
			var sY:Number = speed * Math.sin(angle);
			for (i = 0; i < Math.abs(sX); i++)
			{
				if (!collide("Solid", x + FP.sign(sX), y))
				{
					x += FP.sign(sX) * Math.min(Math.abs(sX) - i, 1);
				}
				else
				{
					break;
				}
			}
			for (i = 0; i < Math.abs(sY); i++)
			{
				if (!collide("Solid", x, y + FP.sign(sY)))
				{
					y += FP.sign(sY) * Math.min(Math.abs(sY) - i, 1);
				}
				else
				{
					break;
				}
			}
		}
		
		private function spriteValues():void
		{
			var spr:Spritemap = (graphic as Spritemap);
			spr.scaleX = FP.sign(direction) * Math.abs(spr.scaleX);
			spr.rate = 1;
			
			if (!roosted) 
			{
				spr.play("Fly");
			}
			else
			{
				spr.play("Sit");
			}
		}
		
	}

}