package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	/**
	 * ...
	 * @author Time
	 */
	public class Charger extends Entity
	{
		[Embed(source = "data/Charger.png")] private var imgCharger:Class;
		private var sprCharger:Spritemap = new Spritemap(imgCharger, 6, 6);
		
		public static const myScale:Number = Game.scale;
		public static const myLayer:int = 0;
		
		private const hitCounterMax:int = 36;
		private const hitCounterInterval:int = 12;
		private var hitCounter:int = 0;
		private const hitColor:uint = 0x000000;
		private const hitsMax:int = 3;
		private var hits:int = 0;
		
		private const accel:Point = new Point(0.1, 0.2);
		private const friction:Number = 0.025;
		private const maxSpeed:Point = new Point(5, 8);
		private var speed:Point = new Point();
		
		private var direction:int = 1;
		
		private var inAir:Boolean = false;
		
		private const lastJumpWaitTimeMax:int = 10;
		private var lastJumpWaitTime:int = 0;
		
		//private var myLight:Light;
		
		public function Charger(_x:int, _y:int) 
		{
			super(_x, _y, sprCharger);
			sprCharger.centerOO();
			sprCharger.centerOrigin();
			sprCharger.scaleX = myScale;
			sprCharger.scaleY = myScale;
			sprCharger.add("Run", [0, 1, 2], 10);
			sprCharger.add("Stand", [0], 0);
			sprCharger.add("Jump", [1], 0);
			sprCharger.play("Stand");
			
			setHitbox(6 * myScale, 4 * myScale, 3*myScale, myScale);
			type = "Charger";
			
			layer = myLayer;
			
			//myLight = new Light(x, y, 2, 0xFF0000);
			//FP.world.add(myLight);
		}
		
		override public function removed():void
		{
			/*if (myLight) 
			{	
				FP.world.remove(myLight);
			}*/
		}
		
		override public function update():void
		{
			hitStep();
			AI();
			movementValues();
			move();
			spriteValues();
			/*if (myLight)
			{
				myLight.x = x + 3 * Lighting.grid * FP.sign(sprCharger.scaleX);
				myLight.y = y + int(sprCharger.frame == 1) * -Lighting.grid + Lighting.grid;
			}*/
		}
		
		private function hitStep():void
		{
			if (hitCounter > 0)
			{
				if (hitCounter % hitCounterInterval == 0)
				{
					if ((graphic as Image).color != hitColor)
					{
						(graphic as Image).color = hitColor;
					}
					else
					{
						(graphic as Image).color = 0xFFFFFF;
					}
				}
				hitCounter--;
			}
			else
			{
				(graphic as Image).color = 0xFFFFFF;
			}
		}
		
		public function getHurt():void
		{
			if (hitCounter <= 0)
			{
				hits++;
				if (hits >= hitsMax)
				{
					FP.world.remove(this);
				}
				hitCounter = hitCounterMax;
			}
		}
		
		private function AI(range:int = -1, jumpHeight:int = 3):void
		{
			var loiter:Boolean = false;
			var player:Player = FP.world.nearestToEntity("Player", this) as Player;
			
			if (inAir)
			{
				return;
			}
			
			if (player)
			{
				var distanceToPlayer:int = FP.distance(x, y, player.x, player.y);
				var seePlayer:Boolean = !FP.world.collideLine("Solid", x, y, player.x, player.y);
				var inRange:Boolean = distanceToPlayer <= range || range == -1;
				var directionToPlayer:int = FP.sign(player.x - x);
				var playerIsAbove:Boolean = player.y + player.height/2 < y + height / 2;
				var blockOnLeft:Solid = collide("Solid", x - 1, y) as Solid;
				var blockOnRight:Solid = collide("Solid", x + 1, y) as Solid;
				var blockOnBottom:Solid = collide("Solid", x, y + 1) as Solid;
				
				if (seePlayer && inRange)
				{
					speed.x += accel.x * directionToPlayer;
				}
				if (lastJumpWaitTime > 0)
				{
					if (blockOnBottom)
					{
						lastJumpWaitTime--;
					}
				}
				else
				{
					if (playerIsAbove)
					{
						if (directionToPlayer <= 0 && blockOnLeft && blockOnBottom)
						{
							speed.y = -jumpHeight;
							lastJumpWaitTime = lastJumpWaitTimeMax;
						}
						else if (directionToPlayer >= 0 && blockOnRight && blockOnBottom)
						{
							speed.y = -jumpHeight;
							lastJumpWaitTime = lastJumpWaitTimeMax;
						}
					}
				}
			}
		}
		
		private function movementValues():void
		{
			var sign:int = FP.sign(speed.x);
			speed.x -= sign * friction;
			if (FP.sign(speed.x) != sign)
			{
				speed.x = 0;
			}
			
			if (Math.abs(speed.x) > maxSpeed.x)
			{
				speed.x = maxSpeed.x * FP.sign(speed.x);
			}
			
			if (speed.x != 0 && !inAir)
			{
				direction = FP.sign(speed.x);
			}
			
			speed.y += accel.y;
			if (speed.y > maxSpeed.y)
			{
				speed.y = maxSpeed.y;
			}
		}
		
		private function move():void
		{
			var i:int;
			for (i = 0; i < Math.abs(speed.x); i++)
			{
				if (!collide("Solid", x + FP.sign(speed.x), y))
				{
					x += FP.sign(speed.x) * Math.min(Math.abs(speed.x) - i, 1);
				}
				else
				{
					if (Math.abs(speed.x) >= maxSpeed.x/2)
					{
						Game.shake = 20;
						speed.x = -FP.sign(speed.x);
						speed.y = -2;
						inAir = true;
					}
					else
					{
						speed.x = 0;
					}
					break;
				}
			}
			for (i = 0; i < Math.abs(speed.y); i++)
			{
				if (!collide("Solid", x, y + FP.sign(speed.y)))
				{
					y += FP.sign(speed.y) * Math.min(Math.abs(speed.y) - i, 1);
				}
				else
				{
					speed.y = 0;
					if (inAir && speed.x == 0)
					{
						inAir = false;
					}
					break;
				}
			}
		}
		
		private function spriteValues():void
		{
			var spr:Spritemap = (graphic as Spritemap);
			spr.scaleX = FP.sign(direction) * Math.abs(spr.scaleX);
			spr.rate = 1;
			
			if (Math.abs(speed.y) > accel.y)
			{
				spr.play("Jump");
			}
			else
			{
				if (Math.abs(speed.x) > accel.x)
				{
					spr.rate = Math.max(Math.abs(speed.x / maxSpeed.x) * 2, 0.5);
					spr.play("Run");
				}
				else
				{
					spr.play("Stand");
				}
			}
		}
		
	}

}