package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	/**
	 * ...
	 * @author Time
	 */
	public class Eye extends Entity
	{
		[Embed(source = "data/Eye.png")] private var imgEye:Class;
		private var sprEye:Spritemap = new Spritemap(imgEye, 4, 4, animEnd);
		
		public static var myScale:int = Game.scale;
		public static const myLayer:int = 50;
		
		private const hitCounterMax:int = 36;
		private const hitCounterInterval:int = 12;
		private var hitCounter:int = 0;
		private const hitColor:uint = 0x000000;
		private const hitsMax:int = 3;
		private var hits:int = 0;
		
		private const accel:Point = new Point(0.1, 0.2);
		private const friction:Number = 0.025;
		private const maxSpeed:Point = new Point(2, 8);
		private var speed:Point = new Point();
		
		private var direction:int = 1;
		private var roosted:Boolean = true;
		
		private const blinkCounterMax:int = Math.random() * 60 + 90;
		private var blinkCounter:int = blinkCounterMax * Math.random();
		
		public function Eye(_x:int, _y:int) 
		{
			super(_x, _y, sprEye);
			sprEye.centerOO();
			sprEye.scaleX = myScale;
			sprEye.scaleY = myScale;
			
			sprEye.add("Blink", [0, 1, 3, 2, 1], 15);
			sprEye.add("Open", [0]);
			
			sprEye.play("Open");
			
			setHitbox(3 * myScale, 3 * myScale, 3 / 2 * myScale, 3 / 2 * myScale);
			type = "Eye";
			
			layer = myLayer;
		}
		
		override public function update():void
		{
			if (blinkCounter > 0)
			{
				blinkCounter--;
			}
			else
			{
				sprEye.play("Blink");
			}
			
			AI(roosted ? 100 : 400, 2);
			if (!roosted)
			{
				hitStep();
				movementValues();
				move();
			}
			
			sprEye.scaleY = myScale + myScale * Math.abs(speed.y) / 8; 
		}
		
		private function hitStep():void
		{
			if (hitCounter > 0)
			{
				if (hitCounter % hitCounterInterval == 0)
				{
					if ((graphic as Image).color != hitColor)
					{
						(graphic as Image).color = hitColor;
					}
					else
					{
						(graphic as Image).color = 0xFFFFFF;
					}
				}
				hitCounter--;
			}
			else
			{
				(graphic as Image).color = 0xFFFFFF;
			}
		}
		
		public function getHurt():void
		{
			if (roosted)
			{
				roosted = false;
			}
			else if (hitCounter <= 0)
			{
				hits++;
				if (hits >= hitsMax)
				{
					FP.world.remove(this);
				}
				hitCounter = hitCounterMax;
			}
		}
		
		private function AI(range:int = -1, jumpHeight:int=0):void
		{
			var player:Player = FP.world.nearestToEntity("Player", this) as Player;
			
			if (player)
			{
				const randomMovementAngle:Number = Math.PI / 180 * 150;
				
				var distanceToPlayer:int = FP.distance(x, y, player.x, player.y);
				var seePlayer:Boolean = !FP.world.collideLine("Solid", x, y, player.x, player.y);
				var inRange:Boolean = distanceToPlayer <= range || range == -1;
				var directionToPlayer:int = FP.sign(player.x - x);
				var blockOnLeft:Solid = collide("Solid", x - 1, y) as Solid;
				var blockOnRight:Solid = collide("Solid", x + 1, y) as Solid;
				var blockOnBottom:Solid = collide("Solid", x, y + 1) as Solid;
				
				if (seePlayer && inRange)
				{
					if (roosted)
					{
						roosted = false;
					}
					else
					{
						if (blockOnBottom)
						{
							speed.x = maxSpeed.x * directionToPlayer;
							speed.y = -jumpHeight;
						}
					}
				}
			}
		}
		
		private function movementValues():void
		{
			var sign:int = FP.sign(speed.x);
			speed.x -= sign * friction;
			if (FP.sign(speed.x) != sign)
			{
				speed.x = 0;
			}
			
			if (Math.abs(speed.x) > maxSpeed.x)
			{
				speed.x = maxSpeed.x * FP.sign(speed.x);
			}
			
			if (speed.x != 0)
			{
				direction = FP.sign(speed.x);
			}
			
			speed.y += accel.y;
			if (Math.abs(speed.y) > maxSpeed.y)
			{
				speed.y = FP.sign(speed.y) * maxSpeed.y;
			}
		}
		
		private function move():void
		{
			var i:int;
			var sX:Number = speed.x;
			var sY:Number = speed.y;
			for (i = 0; i < Math.abs(sX); i++)
			{
				if (!collide("Solid", x + FP.sign(sX), y))
				{
					x += FP.sign(sX) * Math.min(Math.abs(sX) - i, 1);
				}
				else
				{
					speed.x = 0;
					break;
				}
			}
			for (i = 0; i < Math.abs(sY); i++)
			{
				if (!collide("Solid", x, y + FP.sign(sY)))
				{
					y += FP.sign(sY) * Math.min(Math.abs(sY) - i, 1);
				}
				else
				{
					speed.y = 0;
					break;
				}
			}
		}
		
		public function animEnd():void
		{
			if (sprEye.currentAnim == "Blink")
			{
				blinkCounter = blinkCounterMax;
				sprEye.play("Open");
			}
		}
		
	}

}