package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.FP;
	/**
	 * ...
	 * @author Time
	 */
	public class Player extends Entity
	{
		[Embed(source = "data/Player.png")] private var imgPlayer:Class;
		private var sprPlayer:Image = new Image(imgPlayer);
		
		public static const myScale:Number = Game.scale;
		public static const myLayer:int = -1010;
		
		private const keys:Array = new Array(Key.D, Key.S, Key.A, Key.W);
		private const keyLight:int = Key.SPACE;
		private const accel:Point = new Point(0.2, 0.2);
		private const maxSpeed:Point = new Point(3, 8);
		private const friction:Number = 0.2;
		private const jumpSpeed:int = 5;
		
		private var speed:Point = new Point();
		
		private var direction:int = 1;
		private var myLightRange:Number = Math.PI / 2;
		private var myLightBallSpeed:int = 8;
		
		//The distance that my ranged light will reach
		private var defaultLightRange:int = 25;
		//The number of lights that can be thrown before the player runs out of light.
		private var lightNumber:int = 5;
		
		private var myLight:Light;
		private var myLightBall:Vector.<LightBall> = new Vector.<LightBall>();
		private var myGlobeLight:Light;
		
		public function Player(_x:int, _y:int) 
		{
			super(_x, _y, sprPlayer);
			sprPlayer.centerOO();
			sprPlayer.centerOrigin();
			sprPlayer.scale = Game.scale;
			
			setHitbox(myScale * sprPlayer.width * 3 /4, myScale * sprPlayer.height * 3 /4, myScale * sprPlayer.width * 3/8, myScale * sprPlayer.height*3 / 8);
			type = "Player";
			layer = myLayer;
			
			myLight = new Light(x, y, defaultLightRange, 0xFFFFFF, true, false);
			myLight.angleRange = myLightRange;
			FP.world.add(myLight);
			
			myGlobeLight = new Light(x, y, 8, 0xFFFFFF, true, false);
			FP.world.add(myGlobeLight);
		}
		
		override public function update():void
		{
			var mouseAngle:Number = -FP.angle(x, y, FP.camera.x + Input.mouseX, FP.camera.y + Input.mouseY) / 180 * Math.PI;
			
			if (Input.mousePressed)
			{
				/*if (myLightBall)
				{
					FP.world.remove(myLightBall);
				}*/
				if (myLightBall.length < lightNumber)
				{
					myLightBall.push(new LightBall(x, y, new Point(myLightBallSpeed * Math.cos(mouseAngle), myLightBallSpeed * Math.sin(mouseAngle))));
					FP.world.add(myLightBall[myLightBall.length - 1]);
				}
			}
			if (Input.released(keyLight))
			{
				if (myLight)
				{
					myLight.angleRange = 2 * Math.PI;
					myLight.angle = 0;
					myLight.color = FP.getColorARGB(255, Math.random() * 256, Math.random() * 256, Math.random() * 256);
					myLight.dynamicLight = false;
					myLight.drawRect = true;
				}
				myLight = new Light(x, y, defaultLightRange, 0xFFFFFF, true, false);
				myLight.angleRange = myLightRange;
				FP.world.add(myLight);
			}
			movementValues();
			move();
			if (myLight)
			{
				myLight.x = x;
				myLight.y = y;
				myLight.range = defaultLightRange - (defaultLightRange / lightNumber) * myLightBall.length;
			}
			if (myGlobeLight)
			{
				myGlobeLight.x = x;
				myGlobeLight.y = y;
			}
			sprPlayer.angle = -mouseAngle * 180 / Math.PI - 45;
			myLight.angle = mouseAngle;
		}
		
		public function clearLight(removeLight:LightBall):void
		{
			for (var i:int = 0; i < myLightBall.length; i++)
			{
				if (myLightBall[i] == removeLight)
				{
					FP.world.remove(removeLight);
					myLightBall.splice(i, 1);
				}
			}
		}
		
		private function movementValues():void
		{
			if (Input.check(keys[0]))
			{
				speed.x += accel.x;
			}
			if (Input.check(keys[2]))
			{
				speed.x -= accel.x;
			}
			if (!Input.check(keys[0]) && !Input.check(keys[2]))
			{
				var sign:int = FP.sign(speed.x);
				speed.x -= sign * friction;
				if (FP.sign(speed.x) != sign)
				{
					speed.x = 0;
				}
			}
			if (speed.x != 0)
			{
				direction = FP.sign(speed.x);
			}
			if (Input.pressed(keys[3]))
			{
				if (collide("Solid", x, y + 1))
				{
					speed.y = -jumpSpeed;
				}
				else if (collide("Solid", x + 1, y))
				{
					speed.x = -maxSpeed.x / 2;
					speed.y = -jumpSpeed;
				}
				else if (collide("Solid", x - 1, y))
				{
					speed.x = maxSpeed.x / 2;
					speed.y = -jumpSpeed;
				}
			}
			speed.y += accel.y;
			
			if (Math.abs(speed.x) > maxSpeed.x)
			{
				speed.x = maxSpeed.x * FP.sign(speed.x);
			}
			if (speed.y > maxSpeed.y)
			{
				speed.y = maxSpeed.y;
			}
		}
		
		private function move():void
		{
			var i:int;
			for (i = 0; i < Math.abs(speed.x); i++)
			{
				if (!collide("Solid", x + FP.sign(speed.x), y))
				{
					x += FP.sign(speed.x) * Math.min(Math.abs(speed.x) - i, 1);
				}
				else
				{
					speed.x = 0;
					break;
				}
			}
			for (i = 0; i < Math.abs(speed.y); i++)
			{
				if (!collide("Solid", x, y + FP.sign(speed.y)))
				{
					y += FP.sign(speed.y) * Math.min(Math.abs(speed.y) - i, 1);
				}
				else
				{
					speed.y = 0;
					break;
				}
			}
		}
		
	}

}