package editor
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.text.TextField;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Backdrop;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.World;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import flash.system.System;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.events.MouseEvent;
	import flash.events.Event;
	
	/**
	 * ...
	 * @author Time
	 */
	public class Editor extends World
	{
		[Embed(source = "../data/back.png")] private var _imgBack:Class;
		
		[Embed(source = "../data/Player.png")] public static var _imgPlayer:Class;
		public static var sprPlayer:Image = new Image(_imgPlayer);
		
		[Embed(source = "../data/Charger.png")] public static var _imgCharger:Class;
		public static var sprCharger:Spritemap = new Spritemap(_imgCharger, 6, 6);
		
		[Embed(source = "../data/Bat.png")] public static var _imgBat:Class;
		public static var sprBat:Spritemap = new Spritemap(_imgBat, 4, 4);
		
		[Embed(source = "../data/Stalactite.png")] public static var _imgStalactite:Class;
		public static var sprStalactite:Spritemap = new Spritemap(_imgStalactite, 7, 6);
		
		[Embed(source = "../data/EyeWeed.png")] public static var _imgEyeWeed:Class;
		public static var sprEyeWeed:Spritemap = new Spritemap(_imgEyeWeed, 16, 20);
		
		[Embed(source = "../data/BeamBase.png")] public static var _imgBeamBase:Class;
		public static var sprBeamBase:Image = new Image(_imgBeamBase);
		
		//Define constants
		private const iconToBorderMargin:int = 12;
		private const iconToIconMargin:int = 6;
		private	const iconWidth:int = 40;
		private	const iconHeight:int = 40;
		public static const icons:Array = new Array("Solid", "SolidBack", "Stalactite", "Player", "Charger", "Bat", "EyeWeed", "BeamBase");
		public static const images:Array = new Array(null, null, _imgStalactite, _imgPlayer, _imgCharger, _imgBat, _imgEyeWeed, _imgBeamBase);
		public static const sprites:Array = new Array(null, null, sprStalactite, sprPlayer, sprCharger, sprBat, sprEyeWeed, sprBeamBase);
		public static const scales:Array = new Array(null, null, Stalactite.myScale, Player.myScale, Charger.myScale, Bat.myScale, EyeWeed.myScale, BeamBase.myScale);
		public static const layers:Array = new Array(Solid.myLayer, SolidBack.myLayer, Stalactite.myLayer, Player.myLayer, Charger.myLayer, Bat.myLayer, EyeWeed.myLayer, BeamBase.myLayer);
		//These next two variables control which option of the multiple options is currently selected (i.e. skins and different graphics for an item).
		public static const versionsMax:Array = new Array(Game.blockColor.length, Game.blockColorBack.length, sprStalactite.frameCount, 0, 0, 0, 1, 0);
		public static var version:Array = new Array(0, 0, 0, 0, 0, 0, 1, 0);
		//Key constants
		public static const startKey:int = Key.ENTER;
		public static const deleteKey:int = Key.X;
		//Calculates the number of icons that can be fit into each row.
		private	const iconsPerRow:int = Math.ceil((FP.width - 2 * iconToBorderMargin) / (iconToIconMargin + iconWidth));
		//Determines the height of the entire tab region.
		private	const heightAll:int = Math.ceil(icons.length / iconsPerRow) * iconHeight + (Math.ceil(icons.length / iconsPerRow) - 1) * iconToIconMargin + 2 * iconToBorderMargin;
		
		public static var mousePressed:Boolean = false;
		public static var mouseReleased:Boolean = false;
		//Added an item this frame.
		public static var addedItem:Boolean = false;
		
		//The index of the selected item (-1 if null)
		private var selected:int = -1;
		//Whether or not the player is currently using the mouse to place a Solid or other object that
		//requires extra clicks.
		private var settingValues:Boolean = false;
		//The grid-locked point at which the mouse was pressed
		private var pressedPoint:Point = new Point();
		//The grid-locked point at which the mouse was released
		private var releasedPoint:Point = new Point();
		//The grid-locked point at the mouse position
		private var currentPoint:Point = new Point();
		
		//The XML of the level being edited.
		private var currentLevel:XML;
		
		//This TextField will contain any loadable level code.
		private var loadLevel:TextField = new TextField();
		//The XML brought in by the text field.
		private var loadLevelXML:XML;
		//The horizontal margin of the text field to the edge of the sceen.
		private var loadLevelWMargin:int = 6;
		//The vertical margin of the text field to the edge of the screen.
		private var loadLevelHMargin:int = 42;
		//Whether or not the player is loading the level or not.
		private var loadLevelOpen:Boolean = false;
		//The key press that opens the load level option.
		private var loadLevelKey:int = Key.L;
		//Whether or not the level loader is being hovered over by the mouse.
		private var loadLevelHovering:Boolean = false;
		//The button for opening and closing the level loader.
		private var loadLevelButtonText:Text = new Text("Load");
		private var loadLevelButtonMargin:int = 6;
		
		//Used to display the beam for the BeamBase.
		private var yTo:int = 0;
		
		public function Editor(cl:XML = null) 
		{
			//FP.console.enable();
			super();
			currentLevel = cl;
		}
		override public function begin():void
		{
			super.begin();
			
			FP.camera = new Point();
			
			addGraphic(new Backdrop(_imgBack), 1000);
			
			if (currentLevel)
			{
				EditorFunctions.readEditorXML(currentLevel);
			}
			
			for (var i:int = 0; i < sprites.length; i++)
			{
				if (sprites[i])
				{
					sprites[i].scale = scales[i];
					sprites[i].centerOO();
					sprites[i].x = Math.floor(sprites[i].x * sprites[i].scale / Editor.scales[i]) * Editor.scales[i] / sprites[i].scale;
				}
			}
			
			FP.engine.addChild(loadLevel);
			loadLevel.autoSize = TextFieldAutoSize.LEFT;
			loadLevel.type = TextFieldType.INPUT;
			if (currentLevel)
			{
				loadLevel.text = currentLevel.toString();
			}
			else
			{
				loadLevel.text = "Paste level XML here\nthen click 'Load' to load it into the editor.\n\nPress '1' to try out a default level.";
			}
			loadLevel.mouseWheelEnabled = true;
			loadLevel.multiline = true;
			loadLevel.background = true;
			loadLevel.backgroundColor = 0xFFFFFFFF;
			loadLevel.useRichTextClipboard = false;
		}
		
		public function changeTextMouse(e:MouseEvent):void
		{
			//loadLevel.text = "";
			loadLevel.setSelection(0, loadLevel.length);
		}
		
		override public function end():void
		{
			FP.engine.removeChild(loadLevel);
		}
		
		override public function update():void
		{
			super.update();
			addedItem = false;
			mousePressed = Input.mousePressed;
			mouseReleased = Input.mouseReleased;
			
			if (Input.released(Key.DIGIT_1))
			{
				loadLevel.text = Levels.getXML(0).toString();
				loadLevelXML = new XML(loadLevel.text);
				EditorFunctions.readEditorXML(loadLevelXML);
			}
			if (Input.released(Key.DIGIT_2))
			{
				loadLevel.text = Levels.getXML(1).toString();
				loadLevelXML = new XML(loadLevel.text);
				EditorFunctions.readEditorXML(loadLevelXML);
			}
			
			if (Input.mouseX >= loadLevelButtonText.x && Input.mouseX < loadLevelButtonText.x + loadLevelButtonText.width + 2 * loadLevelButtonMargin && 
								   Input.mouseY >= loadLevelButtonText.y && Input.mouseY < loadLevelButtonText.y + loadLevelButtonText.height+ 2 * loadLevelButtonMargin)
			{
				loadLevelHovering = true;
				
				if (mouseReleased)
				{
					loadLevelOpen = !loadLevelOpen;
					
					if (loadLevelOpen)
					{
						if (currentLevel)
						{
							//loadLevel.text = currentLevel.toString();
						}
					}
					else
					{
						try
						{
							loadLevelXML = new XML(loadLevel.text);
						}
						catch (e:Error)
						{
							loadLevelOpen = true;
							loadLevel.text = "Failure to parse - please try again.";
							loadLevelXML = null;
						}
						if (loadLevelXML != null)
						{
							EditorFunctions.readEditorXML(loadLevelXML);
						}
					}
				}
			}
			else
			{
				loadLevelHovering = false;
			}
			
			if (loadLevelOpen)
			{				
				loadLevel.x += (FP.camera.x + loadLevelWMargin - loadLevel.x) / 5;
				loadLevel.y += (FP.camera.y + loadLevelHMargin - loadLevel.y) / 5;
				
				loadLevel.alpha = 1;
				loadLevel.type = TextFieldType.INPUT;
				
				if (!loadLevel.hasEventListener(MouseEvent.CLICK))
				{
					loadLevel.addEventListener(MouseEvent.CLICK, changeTextMouse);
				}
			}
			else
			{
				loadLevel.x = FP.camera.x + FP.width + loadLevelWMargin;
				loadLevel.y = FP.camera.y;
				
				if (loadLevel.hasEventListener(MouseEvent.CLICK))
				{
					loadLevel.removeEventListener(MouseEvent.CLICK, changeTextMouse);
				}
			}
			
			if (mousePressed)
			{
				pressedPoint = new Point(Math.floor(Input.mouseX / Game.scale) * Game.scale, Math.floor(Input.mouseY / Game.scale) * Game.scale);
			}
			if (mouseReleased)
			{
				releasedPoint = new Point(Math.floor(Input.mouseX / Game.scale) * Game.scale, Math.floor(Input.mouseY / Game.scale) * Game.scale);
			}
			currentPoint = new Point(Math.floor(Input.mouseX / Game.scale) * Game.scale, Math.floor(Input.mouseY / Game.scale) * Game.scale);
			//Negate the selected item for prep in the next cycle of items if the player has clicked and is in the tab region.
			if (!Input.check(deleteKey) && !loadLevelHovering && pressedPoint.y < FP.camera.y + FP.height - heightAll)
			{
				if (mousePressed && Input.mouseY > FP.camera.y + FP.height - heightAll)
				{
					selected = -1;
				}
				if (mouseReleased && selected != -1)
				{
					switch(icons[selected])
					{
						case "Solid":
							if (releasedPoint.x > pressedPoint.x && releasedPoint.y > pressedPoint.y)
							{
								add(new Item(pressedPoint.x, pressedPoint.y, selected, releasedPoint.x - pressedPoint.x, releasedPoint.y - pressedPoint.y, version[selected]));
							}
							break;
						case "SolidBack":
							if (releasedPoint.x > pressedPoint.x && releasedPoint.y > pressedPoint.y)
							{
								add(new Item(pressedPoint.x, pressedPoint.y, selected, releasedPoint.x - pressedPoint.x, releasedPoint.y - pressedPoint.y, version[selected]));
							}
							break;
						case "Stalactite": 
							add(new Item(releasedPoint.x, releasedPoint.y, selected, version[selected])); 
							break;
						case "Bat": 
							add(new Item(releasedPoint.x, releasedPoint.y - scales[selected], selected)); 
							break;
						default: add(new Item(releasedPoint.x, releasedPoint.y, selected));
					}
				}
			}
			
			if (Input.released(startKey))
			{
				var cl:XML = EditorFunctions.makeXML();
				//System.setClipboard(cl.toString());
				FP.world = new Game(cl);
			}
		}
		override public function render():void
		{
			super.render();
			loadLevelButtonText.color = 0x000000;
			loadLevelButtonText.x = FP.camera.x + loadLevelButtonMargin;
			loadLevelButtonText.y = FP.camera.y + loadLevelButtonMargin;
			Draw.rect(loadLevelButtonText.x, loadLevelButtonText.y, 
					  loadLevelButtonText.width + 2 * loadLevelButtonMargin, loadLevelButtonText.height + 2 * loadLevelButtonMargin, 0xFFCCCC, 1);
			loadLevelButtonText.render(new Point(loadLevelButtonText.x+1, loadLevelButtonText.y+1), FP.camera);
			
			//drawGrid(Game.scale);
			
			//Draws the current selection onto the screen.
			if (!Input.check(deleteKey))
			{
				switch(icons[selected])
				{
					case "Solid":
						//This will draw the yellow block where a Solid would be placed on release.
						if(Input.mouseDown && currentPoint.x - pressedPoint.x > 0 && currentPoint.y - pressedPoint.y > 0)
						{
							Draw.rect(pressedPoint.x, pressedPoint.y, currentPoint.x - pressedPoint.x, currentPoint.y - pressedPoint.y, 0xFFFF00, 0.4);
						}
						break;
					case "SolidBack":
						//This will draw the yellow block where a Solid would be placed on release.
						if(Input.mouseDown && currentPoint.x - pressedPoint.x > 0 && currentPoint.y - pressedPoint.y > 0)
						{
							Draw.rect(pressedPoint.x, pressedPoint.y, currentPoint.x - pressedPoint.x, currentPoint.y - pressedPoint.y, 0x888800, 0.4);
						}
						break;
					case "Bat":
						if (sprites[selected])
						{
							sprites[selected].scale = scales[selected];
							sprites[selected].render(new Point(currentPoint.x, currentPoint.y - scales[selected]), FP.camera);
						}
						break;
					case "BeamBase":
						if (sprites[selected])
						{
							yTo = currentPoint.y;
							while (!collideLine("Solid", currentPoint.x, currentPoint.y, currentPoint.x, yTo - scales[selected]) && yTo > 0 && yTo <= FP.height)
							{
								yTo -= Lighting.grid;
							}
							Draw.rect(currentPoint.x, yTo, scales[selected], currentPoint.y - yTo, 0xFF0000, 0.8);
							sprites[selected].scaleY = -sprites[selected].scaleY;
							sprites[selected].render(new Point(currentPoint.x, yTo + scales[selected]), FP.camera);
							sprites[selected].scaleY = -sprites[selected].scaleY;
							sprites[selected].scale = scales[selected];
							sprites[selected].render(new Point(currentPoint.x, currentPoint.y), FP.camera);
						}
						break;
					default:
						if (sprites[selected])
						{
							sprites[selected].scale = scales[selected];
							sprites[selected].render(new Point(currentPoint.x, currentPoint.y), FP.camera);
						}
				}
			}
			
			//Draw the background black rectangle.
			Draw.rect(FP.camera.x, FP.camera.y + FP.height - heightAll, FP.width, heightAll, 0x000000, 0.6);
			drawIcons();
		}
		
		public function drawGrid(v:int):void
		{			
			for (var i:int = FP.camera.x; i < FP.camera.x + FP.width; i += v)
			{
				Draw.linePlus(Math.floor(i / v) * v, FP.camera.y, Math.floor(i / v) * v, FP.height, 0xFFFFFFFF, 0.25);
			}
			for (var j:int = FP.camera.y; j < FP.camera.y + FP.height; j += v)
			{
				Draw.linePlus(FP.camera.x, Math.floor(j / v) * v, FP.width, Math.floor(j / v) * v, 0xFFFFFFFF, 0.25);
			}
		}
		
		public function drawIcons():void
		{
			for (var i:int = 0; i < icons.length; i++)
			{
				//Calculates the upper left corner of the current box.
				var xCorner:int = FP.camera.x + iconToBorderMargin + (i % iconsPerRow) * (iconWidth + iconToIconMargin);
				var yCorner:int = FP.camera.y + FP.height - heightAll + (Math.floor(i / iconsPerRow)) * (iconHeight + iconToIconMargin) + iconToBorderMargin;
				
				//Draws a box around the currently selected item, if any.
				if (selected == i)
				{
					Draw.rectLine(xCorner - iconToIconMargin / 2, yCorner - iconToIconMargin / 2, iconWidth + iconToIconMargin, iconHeight + iconToIconMargin, 0xFFFF00, 0.5);
				}
				
				//Determines what to draw in the selection region depending on which string corresponds to this index in icons[]
				switch(icons[i])
				{
					case "Solid": 
						Draw.rect(xCorner, yCorner, iconWidth, iconHeight, Game.blockColor[version[i]], 1);
						break;
					case "SolidBack": 
						Draw.rect(xCorner, yCorner, iconWidth, iconHeight, Game.blockColorBack[version[i]], 1);
						break;
					case "EyeWeed":
						sprites[i].scale = scales[i] / 4;
						version[i] = 1;
						(sprites[i] as Spritemap).frame = 1;
						sprites[i].render(new Point(xCorner + iconWidth / 2 - scales[i]/2*int(sprites[i].width % 2 == 1), yCorner + iconHeight / 2 - scales[i]/2*int(sprites[i].height % 2 == 1)), FP.camera);
						break;
					default:
						sprites[i].scale = scales[i];
						sprites[i].render(new Point(xCorner + iconWidth / 2 - scales[i]/2*int(sprites[i].width % 2 == 1), yCorner + iconHeight / 2 - scales[i]/2*int(sprites[i].height % 2 == 1)), FP.camera);
				}
				//Uses the mouse position and whether or not it has clicked to determine the selected item.
				if (Input.mouseX >= xCorner && Input.mouseY >= yCorner && Input.mouseX < xCorner + iconWidth && Input.mouseY < yCorner + iconHeight)
				{
					Draw.rect(xCorner - iconToIconMargin / 2, yCorner - iconToIconMargin / 2, iconWidth + iconToIconMargin, iconHeight + iconToIconMargin, 0xFFFF00, 0.25);
					if (mousePressed)
					{
						if (selected == i)
						{
							version[i]++;
							if (version[i] >= versionsMax[i])
							{
								version[i] = 0;
							}
							switch(icons[selected])
							{
								case "Stalactite":
									(sprites[selected] as Spritemap).frame = version[i];
									break;
								default:
							}
						}
						selected = i;
					}
				}
			}
		}
		
	}

}